package jdk.internal.org.objectweb.asm;

import java.io.IOException;
import java.io.InputStream;

public class ClassReader
{
    static final boolean SIGNATURES = true;
    static final boolean ANNOTATIONS = true;
    static final boolean FRAMES = true;
    static final boolean WRITER = true;
    static final boolean RESIZE = true;
    public static final int SKIP_CODE = 1;
    public static final int SKIP_DEBUG = 2;
    public static final int SKIP_FRAMES = 4;
    public static final int EXPAND_FRAMES = 8;
    public final byte[] b;
    private final int[] items;
    private final String[] strings;
    private final int maxStringLength;
    public final int header;
    
    public ClassReader(byte[] array) {
        this(array, 0, array.length);
    }
    
    public ClassReader(byte[] b, int n, int n2) {
        this.b = b;
        if (this.readShort(n + 6) > 52) {
            throw new IllegalArgumentException();
        }
        this.items = new int[this.readUnsignedShort(n + 8)];
        int length = this.items.length;
        this.strings = new String[length];
        int maxStringLength = 0;
        int header = n + 10;
        int n3;
        for (int i = 1; i < length; header += n3, ++i) {
            this.items[i] = header + 1;
            switch (b[header]) {
                case 3:
                case 4:
                case 9:
                case 10:
                case 11:
                case 12:
                case 18: {
                    n3 = 5;
                    break;
                }
                case 5:
                case 6: {
                    n3 = 9;
                    ++i;
                    break;
                }
                case 1: {
                    n3 = 3 + this.readUnsignedShort(header + 1);
                    if (n3 > maxStringLength) {
                        maxStringLength = n3;
                        break;
                    }
                    break;
                }
                case 15: {
                    n3 = 4;
                    break;
                }
                default: {
                    n3 = 3;
                    break;
                }
            }
        }
        this.maxStringLength = maxStringLength;
        this.header = header;
        return;
    }
    
    public int getAccess() {
        return this.readUnsignedShort(this.header);
    }
    
    public String getClassName() {
        return this.readClass(this.header + 2, new char[this.maxStringLength]);
    }
    
    public String getSuperName() {
        return this.readClass(this.header + 4, new char[this.maxStringLength]);
    }
    
    public String[] getInterfaces() {
        int n = this.header + 6;
        int unsignedShort = this.readUnsignedShort(n);
        String[] array = new String[unsignedShort];
        if (unsignedShort > 0) {
            char[] array2 = new char[this.maxStringLength];
            for (int i = 0; i < unsignedShort; ++i) {
                n += 2;
                array[i] = this.readClass(n, array2);
            }
            return array;
        }
        return array;
    }
    
    void copyPool(ClassWriter classWriter) {
        char[] array = new char[this.maxStringLength];
        int length = this.items.length;
        Item[] items = new Item[length];
        Object o;
        int n9;
        for (int i = 1; i < length; n9 = o.hashCode % items.length, o.next = items[n9], items[n9] = o, ++i) {
            int n = this.items[i];
            int n2 = this.b[n - 1];
            o = new Item(i);
            switch (n2) {
                case 9:
                case 10:
                case 11: {
                    int n3 = this.items[this.readUnsignedShort(n + 2)];
                    o.set(n2, this.readClass(n, array), this.readUTF8(n3, array), this.readUTF8(n3 + 2, array));
                    break;
                }
                case 3: {
                    o.set(this.readInt(n));
                    break;
                }
                case 4: {
                    o.set(Float.intBitsToFloat(this.readInt(n)));
                    break;
                }
                case 12: {
                    o.set(n2, this.readUTF8(n, array), this.readUTF8(n + 2, array), null);
                    break;
                }
                case 5: {
                    o.set(this.readLong(n));
                    ++i;
                    break;
                }
                case 6: {
                    o.set(Double.longBitsToDouble(this.readLong(n)));
                    ++i;
                    break;
                }
                case 1: {
                    String s = this.strings[i];
                    if (s == null) {
                        int n4 = this.items[i];
                        String[] strings = this.strings;
                        int n5 = i;
                        String utf = this.readUTF(n4 + 2, this.readUnsignedShort(n4), array);
                        strings[n5] = utf;
                        s = utf;
                    }
                    o.set(n2, s, null, null);
                    break;
                }
                case 15: {
                    int n6 = this.items[this.readUnsignedShort(n + 1)];
                    int n7 = this.items[this.readUnsignedShort(n6 + 2)];
                    o.set(20 + this.readByte(n), this.readClass(n6, array), this.readUTF8(n7, array), this.readUTF8(n7 + 2, array));
                    break;
                }
                case 18: {
                    if (classWriter.bootstrapMethods == null) {
                        this.copyBootstrapMethods(classWriter, items, array);
                    }
                    int n8 = this.items[this.readUnsignedShort(n + 2)];
                    o.set(this.readUTF8(n8, array), this.readUTF8(n8 + 2, array), this.readUnsignedShort(n));
                    break;
                }
                default: {
                    o.set(n2, this.readUTF8(n, array), null, null);
                    break;
                }
            }
        }
        int n10 = this.items[1] - 1;
        classWriter.pool.putByteArray(this.b, n10, this.header - n10);
        classWriter.items = items;
        classWriter.threshold = (int)(0.75 * (double)length);
        classWriter.index = length;
    }
    
    private void copyBootstrapMethods(ClassWriter classWriter, Item[] array, char[] array2) {
        int attributes = this.getAttributes();
        boolean n = false;
        int i = this.readUnsignedShort(attributes);
        while (i > 0) {
            if ("BootstrapMethods".equals(this.readUTF8(attributes + 2, array2))) {
                n = true;
                break;
            }
            attributes += 6 + this.readInt(attributes + 4);
            --i;
        }
        if (n == 0) {
            return;
        }
        int unsignedShort = this.readUnsignedShort(attributes + 8);
        int j = 0;
        int n2 = attributes + 10;
        while (j < unsignedShort) {
            int n3 = n2 - attributes - 10;
            int hashCode = this.readConst(this.readUnsignedShort(n2), array2).hashCode();
            for (int k = this.readUnsignedShort(n2 + 2); k > 0; --k) {
                hashCode ^= this.readConst(this.readUnsignedShort(n2 + 4), array2).hashCode();
                n2 += 2;
            }
            n2 += 4;
            Item o = new Item(j);
            o.set(n3, hashCode & Integer.MAX_VALUE);
            int n4 = o.hashCode % array.length;
            o.next = array[n4];
            array[n4] = o;
            ++j;
        }
        int int1 = this.readInt(attributes + 4);
        ByteVector bootstrapMethods = new ByteVector(int1 + 62);
        bootstrapMethods.putByteArray(this.b, attributes + 10, int1 - 2);
        classWriter.bootstrapMethodsCount = unsignedShort;
        classWriter.bootstrapMethods = bootstrapMethods;
        return;
    }
    
    public ClassReader(InputStream inputStream) throws IOException {
        this(readClass(inputStream, false));
    }
    
    public ClassReader(String s) throws IOException {
        this(readClass(ClassLoader.getSystemResourceAsStream(s.replace('.', '/') + ".class"), true));
    }
    
    private static byte[] readClass(InputStream inputStream, boolean b) throws IOException {
        if (inputStream == null) {
            throw new IOException("Class not found");
        }
        try {
            byte[] b2 = new byte[inputStream.available()];
            int off = 0;
            while (true) {
                int read = inputStream.read(b2, off, b2.length - off);
                if (read == -1) {
                    if (off < b2.length) {
                        byte[] array2 = new byte[off];
                        System.arraycopy(b2, 0, array2, 0, off);
                        b2 = array2;
                    }
                }
                else {
                    off += read;
                    if (off == b2.length) {
                        int read2 = inputStream.read();
                        if (read2 < 0) {
                        }
                        else {
                            byte[] array = new byte[b2.length + 1000];
                            System.arraycopy(b2, 0, array, 0, off);
                            array[off++] = (byte)read2;
                            b2 = array;
                        }
                    }
                    else {
                        continue;
                    }
                }
            }
        }
        finally {
            if (b) {
                inputStream.close();
            }
        }
    }
    
    public void accept(ClassVisitor classVisitor, int n) {
        this.accept(classVisitor, new Attribute[0], n);
    }
    
    public void accept(ClassVisitor classVisitor, Attribute[] attrs, int flags) {
        int header = this.header;
        char[] buffer = new char[this.maxStringLength];
        Context context = new Context();
        context.attrs = attrs;
        context.flags = flags;
        context.buffer = buffer;
        int unsignedShort = this.readUnsignedShort(header);
        String class1 = this.readClass(header + 2, buffer);
        String class2 = this.readClass(header + 4, buffer);
        String[] array = new String[this.readUnsignedShort(header + 6)];
        header += 8;
        for (int i = 0; i < array.length; ++i) {
            array[i] = this.readClass(header, buffer);
            header += 2;
        }
        String utf8 = null;
        String utf9 = null;
        String utf10 = null;
        String class3 = null;
        String utf11 = null;
        String utf12 = null;
        int n = 0;
        int n2 = 0;
        int n3 = 0;
        int n4 = 0;
        int n5 = 0;
        Attribute next = null;
        for (int attributes = this.getAttributes(), j = this.readUnsignedShort(attributes); j > 0; attributes += 6 + this.readInt(attributes + 4), --j) {
            String utf13 = this.readUTF8(attributes + 2, buffer);
            if ("SourceFile".equals(utf13)) {
                utf9 = this.readUTF8(attributes + 8, buffer);
            }
            else if ("InnerClasses".equals(utf13)) {
                n5 = attributes + 8;
            }
            else if ("EnclosingMethod".equals(utf13)) {
                class3 = this.readClass(attributes + 8, buffer);
                int unsignedShort2 = this.readUnsignedShort(attributes + 10);
                if (unsignedShort2 != 0) {
                    utf11 = this.readUTF8(this.items[unsignedShort2], buffer);
                    utf12 = this.readUTF8(this.items[unsignedShort2] + 2, buffer);
                }
            }
            else if ("Signature".equals(utf13)) {
                utf8 = this.readUTF8(attributes + 8, buffer);
            }
            else if ("RuntimeVisibleAnnotations".equals(utf13)) {
                n = attributes + 8;
            }
            else if ("RuntimeVisibleTypeAnnotations".equals(utf13)) {
                n3 = attributes + 8;
            }
            else if ("Deprecated".equals(utf13)) {
                unsignedShort |= 0x20000;
            }
            else if ("Synthetic".equals(utf13)) {
                unsignedShort |= 0x41000;
            }
            else if ("SourceDebugExtension".equals(utf13)) {
                int int1 = this.readInt(attributes + 4);
                utf10 = this.readUTF(attributes + 8, int1, new char[int1]);
            }
            else if ("RuntimeInvisibleAnnotations".equals(utf13)) {
                n2 = attributes + 8;
            }
            else if ("RuntimeInvisibleTypeAnnotations".equals(utf13)) {
                n4 = attributes + 8;
            }
            else if ("BootstrapMethods".equals(utf13)) {
                int[] bootstrapMethods = new int[this.readUnsignedShort(attributes + 8)];
                int k = 0;
                int n6 = attributes + 10;
                while (k < bootstrapMethods.length) {
                    bootstrapMethods[k] = n6;
                    n6 += 2 + this.readUnsignedShort(n6 + 2) << 1;
                    ++k;
                }
                context.bootstrapMethods = bootstrapMethods;
            }
            else {
                Attribute attribute = this.readAttribute(attrs, utf13, attributes + 8, this.readInt(attributes + 4), buffer, -1, (Label[])null);
                if (attribute != null) {
                    attribute.next = next;
                    next = attribute;
                }
            }
        }
        classVisitor.visit(this.readInt(this.items[1] - 7), unsignedShort, class1, utf8, class2, array);
        if ((flags & 0x2) == 0x0 && (utf9 != null || utf10 != null)) {
            classVisitor.visitSource(utf9, utf10);
        }
        if (class3 != null) {
            classVisitor.visitOuterClass(class3, utf11, utf12);
        }
        if (n != 0) {
            int l = this.readUnsignedShort(n);
            int annotationValues = n + 2;
            while (l > 0) {
                annotationValues = this.readAnnotationValues(annotationValues + 2, buffer, true, classVisitor.visitAnnotation(this.readUTF8(annotationValues, buffer), true));
                --l;
            }
        }
        if (n2 != 0) {
            int unsignedShort3 = this.readUnsignedShort(n2);
            int annotationValues2 = n2 + 2;
            while (unsignedShort3 > 0) {
                annotationValues2 = this.readAnnotationValues(annotationValues2 + 2, buffer, true, classVisitor.visitAnnotation(this.readUTF8(annotationValues2, buffer), false));
                --unsignedShort3;
            }
        }
        if (n3 != 0) {
            int unsignedShort4 = this.readUnsignedShort(n3);
            int annotationValues3 = n3 + 2;
            while (unsignedShort4 > 0) {
                int annotationTarget = this.readAnnotationTarget(context, annotationValues3);
                annotationValues3 = this.readAnnotationValues(annotationTarget + 2, buffer, true, classVisitor.visitTypeAnnotation(context.typeRef, context.typePath, this.readUTF8(annotationTarget, buffer), true));
                --unsignedShort4;
            }
        }
        if (n4 != 0) {
            int unsignedShort5 = this.readUnsignedShort(n4);
            int annotationValues4 = n4 + 2;
            while (unsignedShort5 > 0) {
                int annotationTarget2 = this.readAnnotationTarget(context, annotationValues4);
                annotationValues4 = this.readAnnotationValues(annotationTarget2 + 2, buffer, true, classVisitor.visitTypeAnnotation(context.typeRef, context.typePath, this.readUTF8(annotationTarget2, buffer), false));
                --unsignedShort5;
            }
        }
        while (next != null) {
            Attribute next2 = next.next;
            next.next = null;
            classVisitor.visitAttribute(next);
            next = next2;
        }
        if (n5 != 0) {
            for (int n7 = n5 + 2, unsignedShort6 = this.readUnsignedShort(n5); unsignedShort6 > 0; n7 += 8, --unsignedShort6) {
                classVisitor.visitInnerClass(this.readClass(n7, buffer), this.readClass(n7 + 2, buffer), this.readUTF8(n7 + 4, buffer), this.readUnsignedShort(n7 + 6));
            }
        }
        int n8 = this.header + 10 + 2 * array.length;
        for (int unsignedShort7 = this.readUnsignedShort(n8 - 2); unsignedShort7 > 0; --unsignedShort7) {
            n8 = this.readField(classVisitor, context, n8);
        }
        n8 += 2;
        for (int unsignedShort8 = this.readUnsignedShort(n8 - 2); unsignedShort8 > 0; --unsignedShort8) {
            n8 = this.readMethod(classVisitor, context, n8);
        }
        classVisitor.visitEnd();
    }
    
    private int readField(ClassVisitor classVisitor, Context context, int n) {
        char[] buffer = context.buffer;
        int unsignedShort = this.readUnsignedShort(n);
        String utf8 = this.readUTF8(n + 2, buffer);
        String utf9 = this.readUTF8(n + 4, buffer);
        n += 6;
        String utf10 = null;
        int n2 = 0;
        int n3 = 0;
        int n4 = 0;
        int n5 = 0;
        Object o = null;
        Attribute next = null;
        for (int i = this.readUnsignedShort(n); i > 0; n += 6 + this.readInt(n + 4), --i) {
            String utf11 = this.readUTF8(n + 2, buffer);
            if ("ConstantValue".equals(utf11)) {
                int unsignedShort2 = this.readUnsignedShort(n + 8);
                o = ((unsignedShort2 == 0) ? null : this.readConst(unsignedShort2, buffer));
            }
            else if ("Signature".equals(utf11)) {
                utf10 = this.readUTF8(n + 8, buffer);
            }
            else if ("Deprecated".equals(utf11)) {
                unsignedShort |= 0x20000;
            }
            else if ("Synthetic".equals(utf11)) {
                unsignedShort |= 0x41000;
            }
            else if ("RuntimeVisibleAnnotations".equals(utf11)) {
                n2 = n + 8;
            }
            else if ("RuntimeVisibleTypeAnnotations".equals(utf11)) {
                n4 = n + 8;
            }
            else if ("RuntimeInvisibleAnnotations".equals(utf11)) {
                n3 = n + 8;
            }
            else if ("RuntimeInvisibleTypeAnnotations".equals(utf11)) {
                n5 = n + 8;
            }
            else {
                Attribute attribute = this.readAttribute(context.attrs, utf11, n + 8, this.readInt(n + 4), buffer, -1, (Label[])null);
                if (attribute != null) {
                    attribute.next = next;
                    next = attribute;
                }
            }
        }
        n += 2;
        FieldVisitor visitField = classVisitor.visitField(unsignedShort, utf8, utf9, utf10, o);
        if (visitField == null) {
            return n;
        }
        if (n2 != 0) {
            int j = this.readUnsignedShort(n2);
            int annotationValues = n2 + 2;
            while (j > 0) {
                annotationValues = this.readAnnotationValues(annotationValues + 2, buffer, true, visitField.visitAnnotation(this.readUTF8(annotationValues, buffer), true));
                --j;
            }
        }
        if (n3 != 0) {
            int k = this.readUnsignedShort(n3);
            int annotationValues2 = n3 + 2;
            while (k > 0) {
                annotationValues2 = this.readAnnotationValues(annotationValues2 + 2, buffer, true, visitField.visitAnnotation(this.readUTF8(annotationValues2, buffer), false));
                --k;
            }
        }
        if (n4 != 0) {
            int l = this.readUnsignedShort(n4);
            int annotationValues3 = n4 + 2;
            while (l > 0) {
                int annotationTarget = this.readAnnotationTarget(context, annotationValues3);
                annotationValues3 = this.readAnnotationValues(annotationTarget + 2, buffer, true, visitField.visitTypeAnnotation(context.typeRef, context.typePath, this.readUTF8(annotationTarget, buffer), true));
                --l;
            }
        }
        if (n5 != 0) {
            int unsignedShort3 = this.readUnsignedShort(n5);
            int annotationValues4 = n5 + 2;
            while (unsignedShort3 > 0) {
                int annotationTarget2 = this.readAnnotationTarget(context, annotationValues4);
                annotationValues4 = this.readAnnotationValues(annotationTarget2 + 2, buffer, true, visitField.visitTypeAnnotation(context.typeRef, context.typePath, this.readUTF8(annotationTarget2, buffer), false));
                --unsignedShort3;
            }
        }
        while (next != null) {
            Attribute next2 = next.next;
            next.next = null;
            visitField.visitAttribute(next);
            next = next2;
        }
        visitField.visitEnd();
        return n;
    }
    
    private int readMethod(ClassVisitor classVisitor, Context context, int classReaderOffset) {
        char[] buffer = context.buffer;
        context.access = this.readUnsignedShort(classReaderOffset);
        context.name = this.readUTF8(classReaderOffset + 2, buffer);
        context.desc = this.readUTF8(classReaderOffset + 4, buffer);
        classReaderOffset += 6;
        int n = 0;
        int n2 = 0;
        String[] array = null;
        String utf8 = null;
        int n3 = 0;
        int n4 = 0;
        int n5 = 0;
        int n6 = 0;
        int n7 = 0;
        int n8 = 0;
        int n9 = 0;
        int n10 = 0;
        Attribute next = null;
        for (int i = this.readUnsignedShort(classReaderOffset); i > 0; classReaderOffset += 6 + this.readInt(classReaderOffset + 4), --i) {
            String utf9 = this.readUTF8(classReaderOffset + 2, buffer);
            if ("Code".equals(utf9)) {
                if ((context.flags & 0x1) == 0x0) {
                    n = classReaderOffset + 8;
                }
            }
            else if ("Exceptions".equals(utf9)) {
                array = new String[this.readUnsignedShort(classReaderOffset + 8)];
                n2 = classReaderOffset + 10;
                for (int j = 0; j < array.length; ++j) {
                    array[j] = this.readClass(n2, buffer);
                    n2 += 2;
                }
            }
            else if ("Signature".equals(utf9)) {
                utf8 = this.readUTF8(classReaderOffset + 8, buffer);
            }
            else if ("Deprecated".equals(utf9)) {
                context.access |= 0x20000;
            }
            else if ("RuntimeVisibleAnnotations".equals(utf9)) {
                n4 = classReaderOffset + 8;
            }
            else if ("RuntimeVisibleTypeAnnotations".equals(utf9)) {
                n6 = classReaderOffset + 8;
            }
            else if ("AnnotationDefault".equals(utf9)) {
                n8 = classReaderOffset + 8;
            }
            else if ("Synthetic".equals(utf9)) {
                context.access |= 0x41000;
            }
            else if ("RuntimeInvisibleAnnotations".equals(utf9)) {
                n5 = classReaderOffset + 8;
            }
            else if ("RuntimeInvisibleTypeAnnotations".equals(utf9)) {
                n7 = classReaderOffset + 8;
            }
            else if ("RuntimeVisibleParameterAnnotations".equals(utf9)) {
                n9 = classReaderOffset + 8;
            }
            else if ("RuntimeInvisibleParameterAnnotations".equals(utf9)) {
                n10 = classReaderOffset + 8;
            }
            else if ("MethodParameters".equals(utf9)) {
                n3 = classReaderOffset + 8;
            }
            else {
                Attribute attribute = this.readAttribute(context.attrs, utf9, classReaderOffset + 8, this.readInt(classReaderOffset + 4), buffer, -1, (Label[])null);
                if (attribute != null) {
                    attribute.next = next;
                    next = attribute;
                }
            }
        }
        classReaderOffset += 2;
        MethodVisitor visitMethod = classVisitor.visitMethod(context.access, context.name, context.desc, utf8, array);
        if (visitMethod == null) {
            return classReaderOffset;
        }
        if ((visitMethod instanceof MethodWriter)) {
            MethodWriter methodWriter = (MethodWriter)visitMethod;
            if (methodWriter.cw.cr == this && utf8 == methodWriter.signature) {
                int n11 = 0;
                if (array == null) {
                    n11 = (methodWriter.exceptionCount == 0);
                }
                else if (array.length == methodWriter.exceptionCount) {
                    n11 = 1;
                    int k = array.length - 1;
                    while (k >= 0) {
                        n2 -= 2;
                        if (methodWriter.exceptions[k] != this.readUnsignedShort(n2)) {
                            n11 = 0;
                            break;
                        }
                        --k;
                    }
                }
                if (n11 != 0) {
                    methodWriter.classReaderOffset = classReaderOffset;
                    methodWriter.classReaderLength = classReaderOffset - classReaderOffset;
                    return classReaderOffset;
                }
            }
        }
        if (n3 != 0) {
            for (int l = this.b[n3] & 0xFF, n12 = n3 + 1; l > 0; --l, n12 += 4) {
                visitMethod.visitParameter(this.readUTF8(n12, buffer), this.readUnsignedShort(n12 + 2));
            }
        }
        if (n8 != 0) {
            AnnotationVisitor visitAnnotationDefault = visitMethod.visitAnnotationDefault();
            this.readAnnotationValue(n8, buffer, null, visitAnnotationDefault);
            if (visitAnnotationDefault != null) {
                visitAnnotationDefault.visitEnd();
            }
        }
        if (n4 != 0) {
            int unsignedShort = this.readUnsignedShort(n4);
            int annotationValues = n4 + 2;
            while (unsignedShort > 0) {
                annotationValues = this.readAnnotationValues(annotationValues + 2, buffer, true, visitMethod.visitAnnotation(this.readUTF8(annotationValues, buffer), true));
                --unsignedShort;
            }
        }
        if (n5 != 0) {
            int unsignedShort2 = this.readUnsignedShort(n5);
            int annotationValues2 = n5 + 2;
            while (unsignedShort2 > 0) {
                annotationValues2 = this.readAnnotationValues(annotationValues2 + 2, buffer, true, visitMethod.visitAnnotation(this.readUTF8(annotationValues2, buffer), false));
                --unsignedShort2;
            }
        }
        if (n6 != 0) {
            int unsignedShort3 = this.readUnsignedShort(n6);
            int annotationValues3 = n6 + 2;
            while (unsignedShort3 > 0) {
                int annotationTarget = this.readAnnotationTarget(context, annotationValues3);
                annotationValues3 = this.readAnnotationValues(annotationTarget + 2, buffer, true, visitMethod.visitTypeAnnotation(context.typeRef, context.typePath, this.readUTF8(annotationTarget, buffer), true));
                --unsignedShort3;
            }
        }
        if (n7 != 0) {
            int unsignedShort4 = this.readUnsignedShort(n7);
            int annotationValues4 = n7 + 2;
            while (unsignedShort4 > 0) {
                int annotationTarget2 = this.readAnnotationTarget(context, annotationValues4);
                annotationValues4 = this.readAnnotationValues(annotationTarget2 + 2, buffer, true, visitMethod.visitTypeAnnotation(context.typeRef, context.typePath, this.readUTF8(annotationTarget2, buffer), false));
                --unsignedShort4;
            }
        }
        if (n9 != 0) {
            this.readParameterAnnotations(visitMethod, context, n9, true);
        }
        if (n10 != 0) {
            this.readParameterAnnotations(visitMethod, context, n10, false);
        }
        while (next != null) {
            Attribute next2 = next.next;
            next.next = null;
            visitMethod.visitAttribute(next);
            next = next2;
        }
        if (n != 0) {
            visitMethod.visitCode();
            this.readCode(visitMethod, context, n);
        }
        visitMethod.visitEnd();
        return classReaderOffset;
    }
    
    private void readCode(MethodVisitor methodVisitor, Context context, int i) {
        byte[] b = this.b;
        char[] buffer = context.buffer;
        int unsignedShort = this.readUnsignedShort(i);
        int unsignedShort2 = this.readUnsignedShort(i + 2);
        int int1 = this.readInt(i + 4);
        i += 8;
        int n = i + int1;
        Label[] labels = new Label[int1 + 2];
        context.labels = labels;
        Label[] array = labels;
        this.readLabel(int1 + 1, array);
        while (i < n) {
            int n2 = i - i;
            switch (ClassWriter.TYPE[b[i] & 0xFF]) {
                case 0:
                case 4: {
                    ++i;
                    continue;
                }
                case 9: {
                    this.readLabel(n2 + this.readShort(i + 1), array);
                    i += 3;
                    continue;
                }
                case 10: {
                    this.readLabel(n2 + this.readInt(i + 1), array);
                    i += 5;
                    continue;
                }
                case 17: {
                    if ((b[i + 1] & 0xFF) == 0x84) {
                        i += 6;
                        continue;
                    }
                    i += 4;
                    continue;
                }
                case 14: {
                    i = i + 4 - (n2 & 0x3);
                    this.readLabel(n2 + this.readInt(i), array);
                    for (int j = this.readInt(i + 8) - this.readInt(i + 4) + 1; j > 0; i += 4, --j) {
                        this.readLabel(n2 + this.readInt(i + 12), array);
                    }
                    i += 12;
                    continue;
                }
                case 15: {
                    i = i + 4 - (n2 & 0x3);
                    this.readLabel(n2 + this.readInt(i), array);
                    for (int k = this.readInt(i + 4); k > 0; i += 8, --k) {
                        this.readLabel(n2 + this.readInt(i + 12), array);
                    }
                    i += 8;
                    continue;
                }
                case 1:
                case 3:
                case 11: {
                    i += 2;
                    continue;
                }
                case 2:
                case 5:
                case 6:
                case 12:
                case 13: {
                    i += 3;
                    continue;
                }
                case 7:
                case 8: {
                    i += 5;
                    continue;
                }
                default: {
                    i += 4;
                    continue;
                }
            }
        }
        for (int l = this.readUnsignedShort(i); l > 0; i += 8, --l) {
            methodVisitor.visitTryCatchBlock(this.readLabel(this.readUnsignedShort(i + 2), array), this.readLabel(this.readUnsignedShort(i + 4), array), this.readLabel(this.readUnsignedShort(i + 6), array), this.readUTF8(this.items[this.readUnsignedShort(i + 8)], buffer));
        }
        i += 2;
        int[] typeAnnotations = null;
        int[] typeAnnotations2 = null;
        int n3 = 0;
        int n4 = 0;
        int n5 = -1;
        int n6 = -1;
        int n7 = 0;
        int n8 = 0;
        boolean n9 = true;
        int n10 = (context.flags & 0x8) != 0x0;
        int frame = 0;
        int n11 = 0;
        int n12 = 0;
        Context context2 = null;
        Attribute next = null;
        for (int unsignedShort3 = this.readUnsignedShort(i); unsignedShort3 > 0; i += 6 + this.readInt(i + 4), --unsignedShort3) {
            String utf8 = this.readUTF8(i + 2, buffer);
            if ("LocalVariableTable".equals(utf8)) {
                if ((context.flags & 0x2) == 0x0) {
                    n7 = i + 8;
                    int unsignedShort6;
                    int unsignedShort7;
                    Label label2;
                    int n14;
                    Label label3;
                    for (unsignedShort6 = this.readUnsignedShort(i + 8), ; unsignedShort6 > 0; i += 10, --unsignedShort6) {
                        unsignedShort7 = this.readUnsignedShort(i + 10);
                        if (array[unsignedShort7] == null) {
                            label2 = this.readLabel(unsignedShort7, array);
                            label2.status |= 0x1;
                        }
                        n14 = unsignedShort7 + this.readUnsignedShort(i + 12);
                        if (array[n14] == null) {
                            label3 = this.readLabel(n14, array);
                            label3.status |= 0x1;
                        }
                    }
                }
            }
            else if ("LocalVariableTypeTable".equals(utf8)) {
                n8 = i + 8;
            }
            else if ("LineNumberTable".equals(utf8)) {
                if ((context.flags & 0x2) == 0x0) {
                    int unsignedShort4;
                    int unsignedShort5;
                    Label label;
                    for (unsignedShort4 = this.readUnsignedShort(i + 8), ; unsignedShort4 > 0; array[unsignedShort5].line = this.readUnsignedShort(i + 12), i += 4, --unsignedShort4) {
                        unsignedShort5 = this.readUnsignedShort(i + 10);
                        if (array[unsignedShort5] == null) {
                            label = this.readLabel(unsignedShort5, array);
                            label.status |= 0x1;
                        }
                    }
                }
            }
            else if ("RuntimeVisibleTypeAnnotations".equals(utf8)) {
                typeAnnotations = this.readTypeAnnotations(methodVisitor, context, i + 8, true);
                n5 = ((typeAnnotations.length == 0 || this.readByte(typeAnnotations[0]) < 67) ? -1 : this.readUnsignedShort(typeAnnotations[0] + 1));
            }
            else if ("RuntimeInvisibleTypeAnnotations".equals(utf8)) {
                typeAnnotations2 = this.readTypeAnnotations(methodVisitor, context, i + 8, false);
                n6 = ((typeAnnotations2.length == 0 || this.readByte(typeAnnotations2[0]) < 67) ? -1 : this.readUnsignedShort(typeAnnotations2[0] + 1));
            }
            else if ("StackMapTable".equals(utf8)) {
                if ((context.flags & 0x4) == 0x0) {
                    frame = i + 10;
                    n11 = this.readInt(i + 4);
                    n12 = this.readUnsignedShort(i + 8);
                }
            }
            else if ("StackMap".equals(utf8)) {
                if ((context.flags & 0x4) == 0x0) {
                    n9 = false;
                    frame = i + 10;
                    n11 = this.readInt(i + 4);
                    n12 = this.readUnsignedShort(i + 8);
                }
            }
            else {
                for (int n13 = 0; n13 < context.attrs.length; ++n13) {
                    if (context.attrs[n13].type.equals(utf8)) {
                        Attribute read = context.attrs[n13].read(this, i + 8, this.readInt(i + 4), buffer, i - 8, array);
                        if (read != null) {
                            read.next = next;
                            next = read;
                        }
                    }
                }
            }
        }
        i += 2;
        if (frame != 0) {
            context2 = context;
            context2.offset = -1;
            context2.mode = 0;
            context2.localCount = 0;
            context2.localDiff = 0;
            context2.stackCount = 0;
            context2.local = new Object[unsignedShort2];
            context2.stack = new Object[unsignedShort];
            if (!(n10 == 0)) {
                this.getImplicitFrame(context);
            }
            for (; frame < frame + n11 - 2; ++frame) {
                if (b[frame] == 8) {
                    int unsignedShort8 = this.readUnsignedShort(frame + 1);
                    if (unsignedShort8 >= 0 && unsignedShort8 < int1 && (b[i + unsignedShort8] & 0xFF) == 0xBB) {
                        this.readLabel(unsignedShort8, array);
                    }
                }
            }
        }
        while (i < n) {
            int n15 = i - i;
            Label label4 = array[n15];
            if (label4 != null) {
                methodVisitor.visitLabel(label4);
                if ((context.flags & 0x2) == 0x0 && label4.line > 0) {
                    methodVisitor.visitLineNumber(label4.line, label4);
                }
            }
            while (context2 != null && (context2.offset == n15 || context2.offset == -1)) {
                if (context2.offset != -1) {
                    if (n9 == 0 || n10) {
                        methodVisitor.visitFrame(-1, context2.localCount, context2.local, context2.stackCount, context2.stack);
                    }
                    else {
                        methodVisitor.visitFrame(context2.mode, context2.localDiff, context2.local, context2.stackCount, context2.stack);
                    }
                }
                if (n12 > 0) {
                    frame = this.readFrame(frame, (boolean)n9, (boolean)n10, context2);
                    --n12;
                }
                else {
                    context2 = null;
                }
            }
            int n16 = b[i] & 0xFF;
            switch (ClassWriter.TYPE[n16]) {
                case 0: {
                    methodVisitor.visitInsn(n16);
                    ++i;
                    break;
                }
                case 4: {
                    if (n16 > 54) {
                        n16 -= 59;
                        methodVisitor.visitVarInsn(54 + (n16 >> 2), n16 & 0x3);
                    }
                    else {
                        n16 -= 26;
                        methodVisitor.visitVarInsn(21 + (n16 >> 2), n16 & 0x3);
                    }
                    ++i;
                    break;
                }
                case 9: {
                    methodVisitor.visitJumpInsn(n16, array[n15 + this.readShort(i + 1)]);
                    i += 3;
                    break;
                }
                case 10: {
                    methodVisitor.visitJumpInsn(n16 - 33, array[n15 + this.readInt(i + 1)]);
                    i += 5;
                    break;
                }
                case 17: {
                    int n17 = b[i + 1] & 0xFF;
                    if (n17 == 132) {
                        methodVisitor.visitIincInsn(this.readUnsignedShort(i + 2), (int)this.readShort(i + 4));
                        i += 6;
                        break;
                    }
                    methodVisitor.visitVarInsn(n17, this.readUnsignedShort(i + 2));
                    i += 4;
                    break;
                }
                case 14: {
                    i = i + 4 - (n15 & 0x3);
                    int n18 = n15 + this.readInt(i);
                    int int2 = this.readInt(i + 4);
                    int int3 = this.readInt(i + 8);
                    Label[] array2 = new Label[int3 - int2 + 1];
                    i += 12;
                    for (int n19 = 0; n19 < array2.length; ++n19) {
                        array2[n19] = array[n15 + this.readInt(i)];
                        i += 4;
                    }
                    methodVisitor.visitTableSwitchInsn(int2, int3, array[n18], array2);
                    break;
                }
                case 15: {
                    i = i + 4 - (n15 & 0x3);
                    int n20 = n15 + this.readInt(i);
                    int int4 = this.readInt(i + 4);
                    int[] array3 = new int[int4];
                    Label[] array4 = new Label[int4];
                    i += 8;
                    for (int n21 = 0; n21 < int4; ++n21) {
                        array3[n21] = this.readInt(i);
                        array4[n21] = array[n15 + this.readInt(i + 4)];
                        i += 8;
                    }
                    methodVisitor.visitLookupSwitchInsn(array[n20], array3, array4);
                    break;
                }
                case 3: {
                    methodVisitor.visitVarInsn(n16, b[i + 1] & 0xFF);
                    i += 2;
                    break;
                }
                case 1: {
                    methodVisitor.visitIntInsn(n16, (int)b[i + 1]);
                    i += 2;
                    break;
                }
                case 2: {
                    methodVisitor.visitIntInsn(n16, (int)this.readShort(i + 1));
                    i += 3;
                    break;
                }
                case 11: {
                    methodVisitor.visitLdcInsn(this.readConst(b[i + 1] & 0xFF, buffer));
                    i += 2;
                    break;
                }
                case 12: {
                    methodVisitor.visitLdcInsn(this.readConst(this.readUnsignedShort(i + 1), buffer));
                    i += 3;
                    break;
                }
                case 6:
                case 7: {
                    int n22 = this.items[this.readUnsignedShort(i + 1)];
                    int n23 = b[n22 - 1] == 11;
                    String class1 = this.readClass(n22, buffer);
                    int n24 = this.items[this.readUnsignedShort(n22 + 2)];
                    String utf9 = this.readUTF8(n24, buffer);
                    String utf10 = this.readUTF8(n24 + 2, buffer);
                    if (n16 < 182) {
                        methodVisitor.visitFieldInsn(n16, class1, utf9, utf10);
                    }
                    else {
                        methodVisitor.visitMethodInsn(n16, class1, utf9, utf10, (boolean)n23);
                    }
                    if (n16 == 185) {
                        i += 5;
                        break;
                    }
                    i += 3;
                    break;
                }
                case 8: {
                    int n25 = this.items[this.readUnsignedShort(i + 1)];
                    int n26 = context.bootstrapMethods[this.readUnsignedShort(n25)];
                    Handle handle = (Handle)this.readConst(this.readUnsignedShort(n26), buffer);
                    int unsignedShort9 = this.readUnsignedShort(n26 + 2);
                    Object[] array5 = new Object[unsignedShort9];
                    n26 += 4;
                    for (int n27 = 0; n27 < unsignedShort9; ++n27) {
                        array5[n27] = this.readConst(this.readUnsignedShort(n26), buffer);
                        n26 += 2;
                    }
                    int n28 = this.items[this.readUnsignedShort(n25 + 2)];
                    methodVisitor.visitInvokeDynamicInsn(this.readUTF8(n28, buffer), this.readUTF8(n28 + 2, buffer), handle, array5);
                    i += 5;
                    break;
                }
                case 5: {
                    methodVisitor.visitTypeInsn(n16, this.readClass(i + 1, buffer));
                    i += 3;
                    break;
                }
                case 13: {
                    methodVisitor.visitIincInsn(b[i + 1] & 0xFF, (int)b[i + 2]);
                    i += 3;
                    break;
                }
                default: {
                    methodVisitor.visitMultiANewArrayInsn(this.readClass(i + 1, buffer), b[i + 3] & 0xFF);
                    i += 4;
                    break;
                }
            }
            while (typeAnnotations != null && n3 < typeAnnotations.length && n5 <= n15) {
                if (n5 == n15) {
                    int annotationTarget = this.readAnnotationTarget(context, typeAnnotations[n3]);
                    this.readAnnotationValues(annotationTarget + 2, buffer, true, methodVisitor.visitInsnAnnotation(context.typeRef, context.typePath, this.readUTF8(annotationTarget, buffer), true));
                }
                n5 = ((++n3 >= typeAnnotations.length || this.readByte(typeAnnotations[n3]) < 67) ? -1 : this.readUnsignedShort(typeAnnotations[n3] + 1));
            }
            while (typeAnnotations2 != null && n4 < typeAnnotations2.length && n6 <= n15) {
                if (n6 == n15) {
                    int annotationTarget2 = this.readAnnotationTarget(context, typeAnnotations2[n4]);
                    this.readAnnotationValues(annotationTarget2 + 2, buffer, true, methodVisitor.visitInsnAnnotation(context.typeRef, context.typePath, this.readUTF8(annotationTarget2, buffer), false));
                }
                n6 = ((++n4 >= typeAnnotations2.length || this.readByte(typeAnnotations2[n4]) < 67) ? -1 : this.readUnsignedShort(typeAnnotations2[n4] + 1));
            }
        }
        if (array[int1] != null) {
            methodVisitor.visitLabel(array[int1]);
        }
        if ((context.flags & 0x2) == 0x0 && n7 != 0) {
            int[] array6 = null;
            if (n8 != 0) {
                i = n8 + 2;
                array6 = new int[this.readUnsignedShort(n8) * 3];
                for (int length = array6.length; length > 0; array6[--length] = i + 6, array6[--length] = this.readUnsignedShort(i + 8), array6[--length] = this.readUnsignedShort(i), i += 10) {}
            }
            i = n7 + 2;
            for (int unsignedShort10 = this.readUnsignedShort(n7); unsignedShort10 > 0; i += 10, --unsignedShort10) {
                int unsignedShort11 = this.readUnsignedShort(i);
                int unsignedShort12 = this.readUnsignedShort(i + 2);
                int unsignedShort13 = this.readUnsignedShort(i + 8);
                String utf11 = null;
                if (array6 != null) {
                    int n29 = 0;
                    while (n29 < array6.length) {
                        if (array6[n29] == unsignedShort11 && array6[n29 + 1] == unsignedShort13) {
                            utf11 = this.readUTF8(array6[n29 + 2], buffer);
                            break;
                        }
                        n29 += 3;
                    }
                }
                methodVisitor.visitLocalVariable(this.readUTF8(i + 4, buffer), this.readUTF8(i + 6, buffer), utf11, array[unsignedShort11], array[unsignedShort11 + unsignedShort12], unsignedShort13);
            }
        }
        if (typeAnnotations != null) {
            for (int n30 = 0; n30 < typeAnnotations.length; ++n30) {
                if (this.readByte(typeAnnotations[n30]) >> 1 == 32) {
                    int annotationTarget3 = this.readAnnotationTarget(context, typeAnnotations[n30]);
                    this.readAnnotationValues(annotationTarget3 + 2, buffer, true, methodVisitor.visitLocalVariableAnnotation(context.typeRef, context.typePath, context.start, context.end, context.index, this.readUTF8(annotationTarget3, buffer), true));
                }
            }
        }
        if (typeAnnotations2 != null) {
            for (int n31 = 0; n31 < typeAnnotations2.length; ++n31) {
                if (this.readByte(typeAnnotations2[n31]) >> 1 == 32) {
                    int annotationTarget4 = this.readAnnotationTarget(context, typeAnnotations2[n31]);
                    this.readAnnotationValues(annotationTarget4 + 2, buffer, true, methodVisitor.visitLocalVariableAnnotation(context.typeRef, context.typePath, context.start, context.end, context.index, this.readUTF8(annotationTarget4, buffer), false));
                }
            }
        }
        while (next != null) {
            Attribute next2 = next.next;
            next.next = null;
            methodVisitor.visitAttribute(next);
            next = next2;
        }
        methodVisitor.visitMaxs(unsignedShort, unsignedShort2);
    }
    
    private int[] readTypeAnnotations(MethodVisitor methodVisitor, Context context, int n, boolean b) {
        char[] buffer = context.buffer;
        int[] array = new int[this.readUnsignedShort(n)];
        n += 2;
        for (int i = 0; i < array.length; ++i) {
            array[i] = n;
            int int1 = this.readInt(n);
            switch (int1 >>> 24) {
                case 0:
                case 1:
                case 22: {
                    n += 2;
                    break;
                }
                case 19:
                case 20:
                case 21: {
                    ++n;
                    break;
                }
                case 64:
                case 65: {
                    for (int j = this.readUnsignedShort(n + 1); j > 0; n += 6, --j) {
                        int unsignedShort = this.readUnsignedShort(n + 3);
                        int unsignedShort2 = this.readUnsignedShort(n + 5);
                        this.readLabel(unsignedShort, context.labels);
                        this.readLabel(unsignedShort + unsignedShort2, context.labels);
                    }
                    n += 3;
                    break;
                }
                case 71:
                case 72:
                case 73:
                case 74:
                case 75: {
                    n += 4;
                    break;
                }
                default: {
                    n += 3;
                    break;
                }
            }
            int byte1 = this.readByte(n);
            if (int1 >>> 24 == 66) {
                Object o = (byte1 == 0) ? null : new TypePath(this.b, n);
                n += 1 + 2 * byte1;
                n = this.readAnnotationValues(n + 2, buffer, true, methodVisitor.visitTryCatchAnnotation(int1, o, this.readUTF8(n, buffer), b));
            }
            else {
                n = this.readAnnotationValues(n + 3 + 2 * byte1, buffer, true, null);
            }
        }
        return array;
    }
    
    private int readAnnotationTarget(Context context, int n) {
        int int1 = this.readInt(n);
        int typeRef;
        switch (int1 >>> 24) {
            case 0:
            case 1:
            case 22: {
                typeRef = (int1 & 0xFFFF0000);
                n += 2;
                break;
            }
            case 19:
            case 20:
            case 21: {
                typeRef = (int1 & 0xFF000000);
                ++n;
                break;
            }
            case 64:
            case 65: {
                typeRef = (int1 & 0xFF000000);
                int unsignedShort = this.readUnsignedShort(n + 1);
                context.start = new Label[unsignedShort];
                context.end = new Label[unsignedShort];
                context.index = new int[unsignedShort];
                n += 3;
                for (int i = 0; i < unsignedShort; ++i) {
                    int unsignedShort2 = this.readUnsignedShort(n);
                    int unsignedShort3 = this.readUnsignedShort(n + 2);
                    context.start[i] = this.readLabel(unsignedShort2, context.labels);
                    context.end[i] = this.readLabel(unsignedShort2 + unsignedShort3, context.labels);
                    context.index[i] = this.readUnsignedShort(n + 4);
                    n += 6;
                }
                break;
            }
            case 71:
            case 72:
            case 73:
            case 74:
            case 75: {
                typeRef = (int1 & 0xFF0000FF);
                n += 4;
                break;
            }
            default: {
                typeRef = (int1 & ((int1 >>> 24 < 67) ? -256 : -16777216));
                n += 3;
                break;
            }
        }
        int byte1 = this.readByte(n);
        context.typeRef = typeRef;
        context.typePath = ((byte1 == 0) ? null : new TypePath(this.b, n));
        return n + 1 + 2 * byte1;
    }
    
    private void readParameterAnnotations(MethodVisitor methodVisitor, Context context, int annotationValues, boolean b) {
        int n = this.b[annotationValues++] & 0xFF;
        int n2;
        int i;
        for (n2 = Type.getArgumentTypes(context.desc).length - n, i = 0; i < n2; ++i) {
            AnnotationVisitor visitParameterAnnotation = methodVisitor.visitParameterAnnotation(i, "Ljava/lang/Synthetic;", false);
            if (visitParameterAnnotation != null) {
                visitParameterAnnotation.visitEnd();
            }
        }
        char[] buffer = context.buffer;
        while (i < n + n2) {
            int j = this.readUnsignedShort(annotationValues);
            annotationValues += 2;
            while (j > 0) {
                annotationValues = this.readAnnotationValues(annotationValues + 2, buffer, true, methodVisitor.visitParameterAnnotation(i, this.readUTF8(annotationValues, buffer), b));
                --j;
            }
            ++i;
        }
    }
    
    private int readAnnotationValues(int n, char[] array, boolean b, AnnotationVisitor annotationVisitor) {
        int i = this.readUnsignedShort(n);
        n += 2;
        if (b) {
            while (i > 0) {
                n = this.readAnnotationValue(n + 2, array, this.readUTF8(n, array), annotationVisitor);
                --i;
            }
        }
        else {
            while (i > 0) {
                n = this.readAnnotationValue(n, array, null, annotationVisitor);
                --i;
            }
        }
        if (annotationVisitor != null) {
            annotationVisitor.visitEnd();
        }
        return n;
    }
    
    private int readAnnotationValue(int n, char[] array, String s, AnnotationVisitor annotationVisitor) {
        if (annotationVisitor == null) {
            switch (this.b[n] & 0xFF) {
                case 101: {
                    return n + 5;
                }
                case 64: {
                    return this.readAnnotationValues(n + 3, array, true, null);
                }
                case 91: {
                    return this.readAnnotationValues(n + 1, array, false, null);
                }
                default: {
                    return n + 3;
                }
            }
        }
        else {
            switch (this.b[n++] & 0xFF) {
                case 68:
                case 70:
                case 73:
                case 74: {
                    annotationVisitor.visit(s, this.readConst(this.readUnsignedShort(n), array));
                    n += 2;
                    break;
                }
                case 66: {
                    annotationVisitor.visit(s, Byte.valueOf((byte)this.readInt(this.items[this.readUnsignedShort(n)])));
                    n += 2;
                    break;
                }
                case 90: {
                    annotationVisitor.visit(s, (this.readInt(this.items[this.readUnsignedShort(n)]) == 0) ? Boolean.FALSE : Boolean.TRUE);
                    n += 2;
                    break;
                }
                case 83: {
                    annotationVisitor.visit(s, Short.valueOf((short)this.readInt(this.items[this.readUnsignedShort(n)])));
                    n += 2;
                    break;
                }
                case 67: {
                    annotationVisitor.visit(s, Character.valueOf((char)this.readInt(this.items[this.readUnsignedShort(n)])));
                    n += 2;
                    break;
                }
                case 115: {
                    annotationVisitor.visit(s, this.readUTF8(n, array));
                    n += 2;
                    break;
                }
                case 101: {
                    annotationVisitor.visitEnum(s, this.readUTF8(n, array), this.readUTF8(n + 2, array));
                    n += 4;
                    break;
                }
                case 99: {
                    annotationVisitor.visit(s, Type.getType(this.readUTF8(n, array)));
                    n += 2;
                    break;
                }
                case 64: {
                    n = this.readAnnotationValues(n + 2, array, true, annotationVisitor.visitAnnotation(s, this.readUTF8(n, array)));
                    break;
                }
                case 91: {
                    int unsignedShort = this.readUnsignedShort(n);
                    n += 2;
                    if (unsignedShort == 0) {
                        return this.readAnnotationValues(n - 2, array, false, annotationVisitor.visitArray(s));
                    }
                    switch (this.b[n++] & 0xFF) {
                        case 66: {
                            byte[] array2 = new byte[unsignedShort];
                            for (int i = 0; i < unsignedShort; ++i) {
                                array2[i] = (byte)this.readInt(this.items[this.readUnsignedShort(n)]);
                                n += 3;
                            }
                            annotationVisitor.visit(s, array2);
                            --n;
                            break;
                        }
                        case 90: {
                            boolean[] array3 = new boolean[unsignedShort];
                            for (int j = 0; j < unsignedShort; ++j) {
                                array3[j] = (this.readInt(this.items[this.readUnsignedShort(n)]) != 0);
                                n += 3;
                            }
                            annotationVisitor.visit(s, array3);
                            --n;
                            break;
                        }
                        case 83: {
                            short[] array4 = new short[unsignedShort];
                            for (int k = 0; k < unsignedShort; ++k) {
                                array4[k] = (short)this.readInt(this.items[this.readUnsignedShort(n)]);
                                n += 3;
                            }
                            annotationVisitor.visit(s, array4);
                            --n;
                            break;
                        }
                        case 67: {
                            char[] array5 = new char[unsignedShort];
                            for (int l = 0; l < unsignedShort; ++l) {
                                array5[l] = (char)this.readInt(this.items[this.readUnsignedShort(n)]);
                                n += 3;
                            }
                            annotationVisitor.visit(s, array5);
                            --n;
                            break;
                        }
                        case 73: {
                            int[] array6 = new int[unsignedShort];
                            for (int n2 = 0; n2 < unsignedShort; ++n2) {
                                array6[n2] = this.readInt(this.items[this.readUnsignedShort(n)]);
                                n += 3;
                            }
                            annotationVisitor.visit(s, array6);
                            --n;
                            break;
                        }
                        case 74: {
                            long[] array7 = new long[unsignedShort];
                            for (int n3 = 0; n3 < unsignedShort; ++n3) {
                                array7[n3] = this.readLong(this.items[this.readUnsignedShort(n)]);
                                n += 3;
                            }
                            annotationVisitor.visit(s, array7);
                            --n;
                            break;
                        }
                        case 70: {
                            float[] array8 = new float[unsignedShort];
                            for (int n4 = 0; n4 < unsignedShort; ++n4) {
                                array8[n4] = Float.intBitsToFloat(this.readInt(this.items[this.readUnsignedShort(n)]));
                                n += 3;
                            }
                            annotationVisitor.visit(s, array8);
                            --n;
                            break;
                        }
                        case 68: {
                            double[] array9 = new double[unsignedShort];
                            for (int n5 = 0; n5 < unsignedShort; ++n5) {
                                array9[n5] = Double.longBitsToDouble(this.readLong(this.items[this.readUnsignedShort(n)]));
                                n += 3;
                            }
                            annotationVisitor.visit(s, array9);
                            --n;
                            break;
                        }
                        default: {
                            n = this.readAnnotationValues(n - 3, array, false, annotationVisitor.visitArray(s));
                            break;
                        }
                    }
                    break;
                }
            }
            return n;
        }
    }
    
    private void getImplicitFrame(Context context) {
        String desc = context.desc;
        Object[] local = context.local;
        int localCount = 0;
        if ((context.access & 0x8) == 0x0) {
            if ("<init>".equals(context.name)) {
                local[localCount++] = Opcodes.UNINITIALIZED_THIS;
            }
            else {
                local[localCount++] = this.readClass(this.header + 2, context.buffer);
            }
        }
        int index = 1;
        while (true) {
            switch (desc.charAt(index++)) {
                case 'B':
                case 'C':
                case 'I':
                case 'S':
                case 'Z': {
                    local[localCount++] = Opcodes.INTEGER;
                    continue;
                }
                case 'F': {
                    local[localCount++] = Opcodes.FLOAT;
                    continue;
                }
                case 'J': {
                    local[localCount++] = Opcodes.LONG;
                    continue;
                }
                case 'D': {
                    local[localCount++] = Opcodes.DOUBLE;
                    continue;
                }
                case '[': {
                    while (desc.charAt(index) == '[') {
                        ++index;
                    }
                    if (desc.charAt(index) == 'L') {
                        ++index;
                        while (desc.charAt(index) != ';') {
                            ++index;
                        }
                    }
                    local[localCount++] = desc.substring(index, ++index);
                    continue;
                }
                case 'L': {
                    while (desc.charAt(index) != ';') {
                        ++index;
                    }
                    local[localCount++] = desc.substring(index + 1, index++);
                    continue;
                }
                default: {
                    context.localCount = localCount;
                    return;
                }
            }
        }
    }
    
    private int readFrame(int n, boolean b, boolean b2, Context context) {
        char[] buffer = context.buffer;
        Label[] labels = context.labels;
        int unsignedShort;
        if (b) {
            unsignedShort = (this.b[n++] & 0xFF);
        }
        else {
            unsignedShort = 255;
            context.offset = -1;
        }
        context.localDiff = 0;
        if (unsignedShort < 64) {
            context.mode = 3;
            context.stackCount = 0;
        }
        else if (unsignedShort < 128) {
            unsignedShort -= 64;
            n = this.readFrameType(context.stack, 0, n, buffer, labels);
            context.mode = 4;
            context.stackCount = 1;
        }
        else {
            unsignedShort = this.readUnsignedShort(n);
            n += 2;
            if (unsignedShort == 247) {
                n = this.readFrameType(context.stack, 0, n, buffer, labels);
                context.mode = 4;
                context.stackCount = 1;
            }
            else if (unsignedShort >= 248 && unsignedShort < 251) {
                context.mode = 2;
                context.localDiff = 251 - unsignedShort;
                context.localCount -= context.localDiff;
                context.stackCount = 0;
            }
            else if (unsignedShort == 251) {
                context.mode = 3;
                context.stackCount = 0;
            }
            else if (unsignedShort < 255) {
                int n4 = b2 ? context.localCount : 0;
                for (int k = unsignedShort - 251; k > 0; --k) {
                    n = this.readFrameType(context.local, n4++, n, buffer, labels);
                }
                context.mode = 1;
                context.localDiff = unsignedShort - 251;
                context.localCount += context.localDiff;
                context.stackCount = 0;
            }
            else {
                context.mode = 0;
                int i = this.readUnsignedShort(n);
                n += 2;
                context.localDiff = i;
                context.localCount = i;
                int n2 = 0;
                while (i > 0) {
                    n = this.readFrameType(context.local, n2++, n, buffer, labels);
                    --i;
                }
                int j = this.readUnsignedShort(n);
                n += 2;
                context.stackCount = j;
                int n3 = 0;
                while (j > 0) {
                    n = this.readFrameType(context.stack, n3++, n, buffer, labels);
                    --j;
                }
            }
        }
        this.readLabel(context.offset += unsignedShort + 1, labels);
        return n;
    }
    
    private int readFrameType(Object[] array, int n, int n2, char[] array2, Label[] array3) {
        switch (this.b[n2++] & 0xFF) {
            case 0: {
                array[n] = Opcodes.TOP;
                break;
            }
            case 1: {
                array[n] = Opcodes.INTEGER;
                break;
            }
            case 2: {
                array[n] = Opcodes.FLOAT;
                break;
            }
            case 3: {
                array[n] = Opcodes.DOUBLE;
                break;
            }
            case 4: {
                array[n] = Opcodes.LONG;
                break;
            }
            case 5: {
                array[n] = Opcodes.NULL;
                break;
            }
            case 6: {
                array[n] = Opcodes.UNINITIALIZED_THIS;
                break;
            }
            case 7: {
                array[n] = this.readClass(n2, array2);
                n2 += 2;
                break;
            }
            default: {
                array[n] = this.readLabel(this.readUnsignedShort(n2), array3);
                n2 += 2;
                break;
            }
        }
        return n2;
    }
    
    protected Label readLabel(int n, Label[] array) {
        if (array[n] == null) {
            array[n] = new Label();
        }
        return array[n];
    }
    
    private int getAttributes() {
        int n = this.header + 8 + this.readUnsignedShort(this.header + 6) * 2;
        for (int i = this.readUnsignedShort(n); i > 0; n += 8, --i) {
            for (int j = this.readUnsignedShort(n + 8); j > 0; --j) {
                n += 6 + this.readInt(n + 12);
            }
        }
        n += 2;
        for (int k = this.readUnsignedShort(n); k > 0; n += 8, --k) {
            for (int l = this.readUnsignedShort(n + 8); l > 0; --l) {
                n += 6 + this.readInt(n + 12);
            }
        }
        return n + 2;
    }
    
    private Attribute readAttribute(Attribute[] array, String anObject, int n, int n2, char[] array2, int n3, Label[] array3) {
        int i = 0;
        while (i < array.length) {
            if (array[i].type.equals(anObject)) {
                return array[i].read(this, n, n2, array2, n3, array3);
            }
            ++i;
        }
        return new Attribute(anObject).read(this, n, n2, null, -1, (Label[])null);
    }
    
    public int getItemCount() {
        return this.items.length;
    }
    
    public int getItem(int n) {
        return this.items[n];
    }
    
    public int getMaxStringLength() {
        return this.maxStringLength;
    }
    
    public int readByte(int n) {
        return this.b[n] & 0xFF;
    }
    
    public int readUnsignedShort(int n) {
        byte[] b = this.b;
        return (b[n] & 0xFF) << 8 | (b[n + 1] & 0xFF);
    }
    
    public short readShort(int n) {
        byte[] b = this.b;
        return (short)((b[n] & 0xFF) << 8 | (b[n + 1] & 0xFF));
    }
    
    public int readInt(int n) {
        byte[] b = this.b;
        return (b[n] & 0xFF) << 24 | (b[n + 1] & 0xFF) << 16 | (b[n + 2] & 0xFF) << 8 | (b[n + 3] & 0xFF);
    }
    
    public long readLong(int n) {
        return (long)this.readInt(n) << 32 | ((long)this.readInt(n + 4) & 0xFFFFFFFFL);
    }
    
    public String readUTF8(int n, char[] array) {
        int unsignedShort = this.readUnsignedShort(n);
        if (n == 0 || unsignedShort == 0) {
            return null;
        }
        String s = this.strings[unsignedShort];
        if (s != null) {
            return s;
        }
        n = this.items[unsignedShort];
        return this.strings[unsignedShort] = this.readUTF(n + 2, this.readUnsignedShort(n), array);
    }
    
    private String readUTF(int i, int n, char[] value) {
        int n2 = i + n;
        byte[] b = this.b;
        int count = 0;
        int n3 = 0;
        int n4 = 0;
        while (i < n2) {
            int n5 = b[i++];
            switch (n3) {
                case 0: {
                    int n6 = n5 & 0xFF;
                    if (n6 < 128) {
                        value[count++] = (char)n6;
                        continue;
                    }
                    if (n6 < 224 && n6 > 191) {
                        n4 = (char)(n6 & 0x1F);
                        n3 = 1;
                        continue;
                    }
                    n4 = (char)(n6 & 0xF);
                    n3 = 2;
                    continue;
                }
                case 1: {
                    value[count++] = (char)(n4 << 6 | (n5 & 0x3F));
                    n3 = 0;
                    continue;
                }
                case 2: {
                    n4 = (char)(n4 << 6 | (n5 & 0x3F));
                    n3 = 1;
                    continue;
                }
            }
        }
        return new String(value, 0, count);
    }
    
    public String readClass(int n, char[] array) {
        return this.readUTF8(this.items[this.readUnsignedShort(n)], array);
    }
    
    public Object readConst(int n, char[] array) {
        int n2 = this.items[n];
        switch (this.b[n2 - 1]) {
            case 3: {
                return Integer.valueOf(this.readInt(n2));
            }
            case 4: {
                return Float.valueOf(Float.intBitsToFloat(this.readInt(n2)));
            }
            case 5: {
                return Long.valueOf(this.readLong(n2));
            }
            case 6: {
                return Double.valueOf(Double.longBitsToDouble(this.readLong(n2)));
            }
            case 7: {
                return Type.getObjectType(this.readUTF8(n2, array));
            }
            case 8: {
                return this.readUTF8(n2, array);
            }
            case 16: {
                return Type.getMethodType(this.readUTF8(n2, array));
            }
            default: {
                int byte1 = this.readByte(n2);
                int[] items = this.items;
                int n3 = items[this.readUnsignedShort(n2 + 1)];
                String class1 = this.readClass(n3, array);
                int n4 = items[this.readUnsignedShort(n3 + 2)];
                return new Handle(byte1, class1, this.readUTF8(n4, array), this.readUTF8(n4 + 2, array));
            }
        }
    }
}
